﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;

[RequireComponent(typeof(LevelConfig))]
public class GameCycleManager : MonoBehaviour
{
    #region Constants

    public enum State
    {
        Invalid,
        Intro,
        Placement,
        LaunchSoldiers,
        OutroFail,
        OutroWin,
        NextLevel,
    }

    #endregion

    #region Fields

    public static GameCycleManager Instance;

    public Camera WorldCamera;

    private LevelConfig _levelConfig;
    private bool nextButtonPressed;

    #endregion

    #region Properties

    public State CurrentState { get; private set; }

    #endregion

    #region UnityMethods

    // Use this for initialization
    void Awake()
    {
        Instance = this;
        _levelConfig = GetComponent<LevelConfig>();
        _levelConfig.ObjectivesCompleteCallBack = AllObjectivesComplete;
    }

    void Start()
    {
        var currentSceneIndex = SceneManager.GetActiveScene().buildIndex;

        if (currentSceneIndex == LevelConfig.LastSceneIndex)
        {
            SetState(State.Placement);
        }

        else
        {
            SetState(State.Intro);
            LevelConfig.LastSceneIndex = currentSceneIndex;
        }
    }

    // Update is called once per frame
    void Update()
    {
        SoldierCounter.SetSolderCount(_levelConfig.GetCurrentNonDeadSoldierCount());
        KillLimitCounter.SetKillCount(_levelConfig.GetCurrentKillLimitCount());
    }

    #endregion

    #region Public Methods

    private void SetState(State newState)
    {
        if (newState != CurrentState)
        {
            StopAllCoroutines();

            switch (newState)
            {
            case State.Intro:
                StartCoroutine(IntroState());
                break;

            case State.Placement:
                StartCoroutine(PlacementState());
                break;

            case State.LaunchSoldiers:
                StartCoroutine(LaunchSoldiersState());
                break;

            case State.OutroFail:
                StartCoroutine(OutroFailState());
                break;

            case State.OutroWin:
                StartCoroutine(OutroWinState());
                break;

            default:
                Debug.Log("GameCycleManager.SetState -> State is invalid (" + newState + ")");
                CurrentState = State.Invalid;
                break;

            }
            CurrentState = newState;
        }
    }

    public void OnNextButtonPressed()
    {
        nextButtonPressed = true;
//        if (CurrentState == State.Intro)
//        {
//            SetState(State.Placement);
//        }
    }

    public void OnGoButtonPressed()
    {
        if (CurrentState == State.Placement)
        {
            SetState(State.LaunchSoldiers);
        }
    }

    public void OnAbortButtonPressed()
    {
        if (CurrentState == State.LaunchSoldiers)
        {
            // TODO : DESTROY SOLDIERS!
            SetState(State.OutroFail);
        }
    }

    public void OnContinueButtonPressed()
    {
        if (CurrentState == State.OutroWin)
        {
            int scene = SceneManager.GetActiveScene().buildIndex;
            SceneManager.LoadScene(scene + 1, LoadSceneMode.Single);
            
            StopAllCoroutines();
        }
    }
    
    public void OnRetryButtonPressed()
    {
        if (CurrentState == State.OutroFail || CurrentState == State.OutroWin)
        {
            int scene = SceneManager.GetActiveScene().buildIndex;
            SceneManager.LoadScene(scene, LoadSceneMode.Single);

            StopAllCoroutines();
        }
    }

    public void OnQuitButtonPressed()
    {
        if (CurrentState == State.OutroFail || CurrentState == State.OutroWin)
        {
            SceneManager.LoadScene(0, LoadSceneMode.Single);

            StopAllCoroutines();
        }
    }

    public void OnInventoryButtonPressed(int itemIndex)
    {
        if (CurrentState == State.Placement)
        {
            Debug.Log("OnInventoryButtonPressed(" + itemIndex + ")");

            _levelConfig.GetItemFromInventory(itemIndex);

            //TODO : GET CONSTRUCTABLE
        }
    }

    #endregion

    #region Private Methods

    private IEnumerator IntroState()
    {
        Time.timeScale = 0;
        UserInterfaceController.Instance.SetState(UserInterfaceController.State.Intro);

        int textPageCount = _levelConfig.IntroText.Length;

        if (_levelConfig.IntroText.Length > 0)
        {
            var textPage = 0;

            while (textPage < textPageCount)
            {
                UserInterfaceController.Instance.SetIntroText(_levelConfig.IntroText[textPage]);

                while (!nextButtonPressed)
                {
                    yield return null;
                }

                nextButtonPressed = false;
                textPage++;
            }
        }

        // TODO: Add intro state behaviour here!

//        while (!Input.GetKeyDown(KeyCode.Space))
//        {
//            yield return null;
//        }

        yield return null;

        SetState(State.Placement);
    }

    private IEnumerator PlacementState()
    {
        Time.timeScale = 0;
        UserInterfaceController.Instance.SetState(UserInterfaceController.State.Placement);

        Debug.Log("GameCycleManager -> PlacementState");

        yield return null;

//        // TODO: Add placement state behaviour here!
//
//        while (!Input.GetKeyDown(KeyCode.Space))
//        {
//            yield return null;
//        }
//
//        yield return null;
//
//        SetState(State.LaunchSoldiers);
    }

    private IEnumerator LaunchSoldiersState()
    {
        UserInterfaceController.Instance.SetState(UserInterfaceController.State.LaunchSoldiers);
        Time.timeScale = 1;

        _levelConfig.SetupForLaunchSoldierState();

        Debug.Log("GameCycleManager -> LaunchSoldiersState");

        //TOdo add correct fail checks here.

        while (!Input.GetKeyDown(KeyCode.Space))
        {
            // Launch Soldier State Game Loop.


            yield return null;
        }

        yield return null;

        SetState(State.OutroFail);
    }

    private IEnumerator OutroFailState()
    {
        UserInterfaceController.Instance.SetState(UserInterfaceController.State.OutroFail);

        // TODO: Ramp down timescale;

        Time.timeScale = 0.25f;

        Debug.Log("GameCycleManager -> OutroFailState");

        // TODO: Add intro state behaviour here!

        while (!Input.GetKeyDown(KeyCode.Space))
        {
            yield return null;
        }

        yield return null;

        int scene = SceneManager.GetActiveScene().buildIndex;
        SceneManager.LoadScene(scene, LoadSceneMode.Single);

        StopAllCoroutines();

        //        SetState(State.Placement);
    }

    private IEnumerator OutroWinState()
    {
        UserInterfaceController.Instance.SetState(UserInterfaceController.State.OutroWin);

        // TODO: Ramp down timescale;

        Time.timeScale = 0.25f;

        Debug.Log("GameCycleManager -> OutroWinState");

        // TODO: Add intro state behaviour here!

        while (!Input.GetKeyDown(KeyCode.Space))
        {
            yield return null;
        }

        yield return null;

        SetState(State.NextLevel);
    }

    private void AllObjectivesComplete()
    {
        if (CurrentState == State.LaunchSoldiers)
        {
            SetState(State.OutroWin);
        }
    }


    #endregion
}
