﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class LevelConfig : MonoBehaviour
{
    public delegate void AllObjectivesCompleteCallBack();

    #region Structs

    [Serializable]
    public class ConstructableInventoryItemDetails
    {
        public string Name;
        public ConstructableItemBase ItemPrefab;
        public int AvailableCount;

        private List<ConstructableItemBase> _unplacedItems;
        private List<ConstructableItemBase> _placedItems;
        private InventoryItemButton _inventoryButton;

        //        public void InitialiseItems(InventoryItemButton inventoryButtom)
        public void InitialiseItems(int inventoryItemIndex)
        {
            _unplacedItems = new List<ConstructableItemBase>(AvailableCount);
            _placedItems = new List<ConstructableItemBase>(AvailableCount);

            for (int i = 0; i < AvailableCount; i++)
            {
                var itemGameObject = Instantiate(ItemPrefab.gameObject);
                itemGameObject.SetActive(false);
                var constructableItem = itemGameObject.GetComponent<ConstructableItemBase>();
                constructableItem.InventoryItemIndex = inventoryItemIndex;
                _unplacedItems.Add(itemGameObject.GetComponent<ConstructableItemBase>());
            }

            _inventoryButton = UserInterfaceController.Instance.GetInventoryItemButton(inventoryItemIndex);
            _inventoryButton.SetButtonImage(ItemPrefab.InventoryButtonSprite);
            _inventoryButton.SetItemCount(_unplacedItems.Count);
        }

        public ConstructableItemBase GetPlaceableItem()
        {
            if (_unplacedItems.Count <= 0)
            {
                return null;
            }

            ConstructableItemBase item = _unplacedItems[0];
            _unplacedItems.Remove(item);
            _placedItems.Add(item);
            _inventoryButton.SetItemCount(_unplacedItems.Count);

            item.gameObject.SetActive(true);
            return item;
        }

        public void ReturnPlacabledItem(ConstructableItemBase item)
        {
            if (!_placedItems.Contains(item))
            {
                return;
            }

            item.gameObject.SetActive(false);

            _placedItems.Remove(item);
            _unplacedItems.Add(item);

            _inventoryButton.SetItemCount(_unplacedItems.Count);
        }
    }

    public struct ConstructableStateData
    {
        public string Name;
        public Vector3 Position;
    }

    #endregion

    #region Fields

    [TextArea(3, 10)] public string[] IntroText = { "BLAH BLAH BLAH BLAH...."};

    public int MaxInventoryItems = 6;
    public ConstructableInventoryItemDetails[] InventoryItems;

    public AllObjectivesCompleteCallBack ObjectivesCompleteCallBack;

    private SpawnPointBase[] _spawnPoints;
    private ObjectiveBase[] _objectives;

    private int _objectiveTotalCount;
    private int _objectiveCompleteCount;

    private ObjectiveSoldierKillCounter[] _objectivesSoldierKillCounters;
    private int _objectivesSoldierKillCounterCount;

    private int _spawnPointCount;

    public static int LastSceneIndex = -1;
    private static List<ConstructableStateData> _lastConstructableStates = new List<ConstructableStateData>();

    #endregion

    #region Unity Methods

    // Use this for initialization
    void Start()
    {
        var killCounterList = new List<ObjectiveSoldierKillCounter>();

        _objectives = FindObjectsOfType<ObjectiveBase>();
        _spawnPoints = FindObjectsOfType<SpawnPointBase>();

        _objectiveTotalCount = _objectives.Length;

        for (int i = 0; i < _objectiveTotalCount; i++)
        {
            _objectives[i].CompleteCallBack += ObjectiveComplete;

            var ObjectiveKillCounterComponent = _objectives[i].GetComponent<ObjectiveSoldierKillCounter>();

            if (ObjectiveKillCounterComponent != null)
            {
                killCounterList.Add(ObjectiveKillCounterComponent);
            }
        }

        _objectivesSoldierKillCounters = killCounterList.ToArray();
        _objectivesSoldierKillCounterCount = _objectivesSoldierKillCounters.Length;

        _spawnPointCount = _spawnPoints.Length;

        var inventoryItemCount = InventoryItems.Length;

        for (int i = 0; i < MaxInventoryItems; i++)
        {
            if (i < inventoryItemCount)
            {
                InventoryItems[i].InitialiseItems(i);
            }
            else
            {
                UserInterfaceController.Instance.GetInventoryItemButton(i).SetItemCount(-9999);
            }
        }
    }

    // Update is called once per frame
    void Update()
    {

    }

    #endregion

    #region Public Methods

    public void SetupForPlacementState()
    {

    }

    public void SetupForLaunchSoldierState()
    {
        _objectiveCompleteCount = 0;

        for (int i = 0; i < _spawnPointCount; i++)
        {
            _spawnPoints[i].StartSpawn();
        }
    }

    public int GetCurrentNonDeadSoldierCount()
    {
        int totalSoldierCount = 0;

        for (int i = 0; i < _spawnPointCount; i++)
        {
            totalSoldierCount += _spawnPoints[i].NonDeadSoldierCount;
        }

        return totalSoldierCount;
    }

    public int GetCurrentKillLimitCount()
    {
        int targetKillCount = 0;
        int totalKillCount = 0;

        for (int i = 0; i < _objectivesSoldierKillCounterCount; i++)
        {
            targetKillCount += _objectivesSoldierKillCounters[i].GetTargetKillCount();
            totalKillCount += _objectivesSoldierKillCounters[i].GetCurrentKillCount();
        }

        return targetKillCount - totalKillCount;
    }

    public void GetItemFromInventory(int itemIndex)
    {
        var activeConstructable = InventoryItems[itemIndex].GetPlaceableItem();

        StartCoroutine(ConstructablePlacementCoroutine(activeConstructable));
    }

    public IEnumerator ConstructablePlacementCoroutine(ConstructableItemBase item)
    {
        bool inputReceived = false;

        while (!inputReceived)
        {
            Vector3 newPos = GameCycleManager.Instance.WorldCamera.ScreenToWorldPoint(Input.mousePosition);
            newPos.z = 0;
            item.transform.position = newPos;

            if (Input.GetMouseButtonDown(0))
            {
                inputReceived = true;
            }
            else if (Input.GetMouseButtonDown(1))
            {
                InventoryItems[item.InventoryItemIndex].ReturnPlacabledItem(item);

                inputReceived = true;
            }

            yield return null;
        }
    }

    #endregion

    #region Private Methods

    private void ObjectiveComplete(ObjectiveBase objective)
    {
        _objectiveCompleteCount++;

        if (_objectiveCompleteCount >= _objectiveTotalCount && ObjectivesCompleteCallBack != null)
        {
            ObjectivesCompleteCallBack.Invoke();
        }
    }

    #endregion
}
